import { __decorate, __metadata, __awaiter, __param } from 'tslib';
import { Input, HostBinding, Directive, EventEmitter, ElementRef, Renderer2, ChangeDetectorRef, Output, HostListener, Component, ChangeDetectionStrategy, ViewEncapsulation, ɵɵdefineInjectable, Injectable, TemplateRef, ContentChild, Optional, ViewChild, Pipe, NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { AbstractControl, FormGroup, FormArray, Validators, FormControl, FormGroupDirective, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { evalToBool } from '@anjuna/core';
import { AnjunaValidators, BaseTranslationPipe, AnjunaCoreModule } from '@anjuna/angular-core';
import { CommonModule } from '@angular/common';

/* @internal */
let BaseSchema = class BaseSchema {
    get hideLabels() {
        return this._hideLabels;
    }
    set hideLabels(value) {
        if (typeof value === 'string') {
            this._hideLabels = evalToBool(value);
        }
        else {
            this._hideLabels = value;
        }
    }
    trackByFn(index) {
        return index;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], BaseSchema.prototype, "hideLabels", null);
__decorate([
    HostBinding('attr.hideLabel'),
    __metadata("design:type", Boolean)
], BaseSchema.prototype, "_hideLabels", void 0);
BaseSchema = __decorate([
    Directive()
], BaseSchema);

class ParsedField {
    constructor(name, fieldDef, requiredFields = [], readonlyFields = [], relationships = []) {
        this.default = fieldDef.default;
        this.description = fieldDef.description;
        this.enforceUnique = fieldDef.uniqueItems;
        this.format = fieldDef.format;
        this.items = fieldDef.items;
        this.label = fieldDef.title || fieldDef.label || name;
        this.maximum = fieldDef.maximum;
        this.maxItems = fieldDef.maxItems;
        this.maxLength = fieldDef.maxLength;
        this.minimum = fieldDef.minimum;
        this.minItems = fieldDef.minItems;
        this.minLength = fieldDef.minLength;
        this.name = name;
        this.nestedDef = fieldDef;
        this.options = fieldDef.enum;
        this.pattern = fieldDef.pattern;
        this.parsedArray = fieldDef.parsedArray;
        this.parsedList = fieldDef.parsedList;
        this.parsedFields = fieldDef.parsedFields;
        this.properties = fieldDef.properties;
        this.readonly = fieldDef.readonly || (readonlyFields ? readonlyFields.includes(name) : false);
        this.required = requiredFields ? requiredFields.includes(name) : false;
        this.subtype = fieldDef.subtype;
        this.type = fieldDef.type;
        this.uoms = fieldDef.UOMs;
        this.uom = fieldDef.UOM;
        this.relationships = relationships;
    }
    toInputType() {
        if (this.format === 'date') {
            return 'date';
        }
        else if (this.options) {
            return 'select';
        }
        else if (this.type === 'integer' || this.type === 'number') {
            return 'number';
        }
        else if (this.type === 'boolean') {
            return 'checkbox';
        }
        else if (this.type === 'array' && this.subtype === 'tier') {
            return 'tier';
        }
        else if (this.type === 'array' && this.items && this.items.type === 'object') {
            return 'table';
        }
        else if (this.type === 'array' && this.items) {
            return 'list';
        }
        else if (this.type === 'object' && this.properties) {
            return 'nested';
        }
        else if (this.type === 'string' && this.subtype === 'area') {
            return 'area';
        }
        else if (this.format === 'uuid' &&
            this.relationships.some((r) => Object.keys(r.fields).includes(this.name))) {
            return 'relationship';
        }
        return 'text';
    }
}

/* @internal */
let SchemaField = class SchemaField extends BaseSchema {
    constructor() {
        super();
        this.isField = true;
    }
};
__decorate([
    Input(),
    __metadata("design:type", ParsedField)
], SchemaField.prototype, "field", void 0);
SchemaField = __decorate([
    Directive(),
    __metadata("design:paramtypes", [])
], SchemaField);

let SchemaFieldComponent = class SchemaFieldComponent extends SchemaField {
    constructor(element, renderer, changeRef) {
        super();
        this.element = element;
        this.renderer = renderer;
        this.changeRef = changeRef;
        this.tierToChange = new EventEmitter();
    }
    _onFieldChange(event) {
        if (this.field.name === 'tierTo') {
            this.tierToChange.emit(this.index);
        }
    }
    ngAfterViewInit() {
        this.renderer.addClass(this.element.nativeElement, this.field.name.split(' ').join('_'));
    }
    handleOnChangeCallback({ detail }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.inputChangeCallback) {
                this.typeaheadOptions = yield this.inputChangeCallback(detail);
                this.changeRef.markForCheck();
            }
        });
    }
};
SchemaFieldComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: ChangeDetectorRef }
];
__decorate([
    Input(),
    __metadata("design:type", AbstractControl)
], SchemaFieldComponent.prototype, "control", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], SchemaFieldComponent.prototype, "index", void 0);
__decorate([
    Input(),
    __metadata("design:type", Function)
], SchemaFieldComponent.prototype, "inputChangeCallback", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], SchemaFieldComponent.prototype, "tierToChange", void 0);
__decorate([
    HostBinding(`class.anj-schema-field`),
    __metadata("design:type", Boolean)
], SchemaFieldComponent.prototype, "isField", void 0);
__decorate([
    HostListener('anjChange', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [CustomEvent]),
    __metadata("design:returntype", void 0)
], SchemaFieldComponent.prototype, "_onFieldChange", null);
SchemaFieldComponent = __decorate([
    Component({
        selector: 'anj-schema-field',
        template: "<ng-container [ngSwitch]=\"field.toInputType()\">\n    <ng-template ngSwitchCase=\"relationship\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-type-ahead (anjQuery)=\"handleOnChangeCallback($event)\" [formControl]=\"control\" display=\"label\">\n                <anj-option *ngFor=\"let option of typeaheadOptions\" [value]=\"option\">\n                    {{option.label}}\n                </anj-option>\n            </anj-type-ahead>\n        </anj-form-group>\n        <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n    </ng-template>\n    <ng-template ngSwitchCase=\"date\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\" [min]=\"field.minimum\"\n                [max]=\"field.maximum\" [attr.readonly]=\"field.readonly\" [attr.required]=\"field.required\">\n                <anj-date-picker-toggle></anj-date-picker-toggle>\n                <anj-date-picker></anj-date-picker>\n            </anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.date?.valid === false\">\n                    {{field.label || field.name}} {{'is not a valid date.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"select\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-select placeholder=\"{{hideLabels ? field.name : null}}\" [formControl]=\"control\"\n                [attr.required]=\"field.required\">\n                <anj-option *ngFor=\"let option of field.options\" [value]=\"option\">{{option}}</anj-option>\n                <ng-template [ngIf]=\"control.errors\">\n                    <anj-form-error>{{'Please select an option.' | anjSchemaTranslate}}</anj-form-error>\n                </ng-template>\n            </anj-select>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"number\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" step=\"1\" type=\"number\" placeholder=\"{{hideLabels ? field.name : null}}\"\n                [min]=\"field.minimum\" [max]=\"field.maximum\" [attr.readonly]=\"field.readonly\"\n                [attr.required]=\"field.required\">\n                <anj-number-stepper></anj-number-stepper>\n            </anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.min\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}}\n                    {{control.errors.min.min}}.\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.max\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}}\n                    {{control.errors.max.max}}.\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"checkbox\">\n        <anj-check-group>\n            <anj-checkbox [formControl]=\"control\">{{field.label}}</anj-checkbox>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n        </anj-check-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"list\">\n        <anj-form-group>\n            <anj-list [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\"\n                [attr.required]=\"field.required\"></anj-list>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.unique\">\n                    {{field.label || field.name}} {{'must not have duplicate values.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.minlength\">\n                    {{field.label || field.name}} {{'must have at least' | anjSchemaTranslate}}\n                    {{control.errors['minlength'].requiredLength}} {{'entries.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"nested\">\n        <anj-schema-nested-field [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\">\n        </anj-schema-nested-field>\n    </ng-template>\n    <ng-template ngSwitchCase=\"table\">\n        <anj-schema-table [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\">\n        </anj-schema-table>\n    </ng-template>\n    <ng-template ngSwitchCase=\"tier\">\n        <anj-schema-tier [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\">\n        </anj-schema-tier>\n    </ng-template>\n    <ng-template ngSwitchCase=\"area\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-textarea [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\"\n                [attr.readonly]=\"field.readonly\" [attr.required]=\"field.required\"></anj-textarea>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.minlength\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}}\n                    {{control.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.maxlength\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}}\n                    {{control.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.pattern\">\n                    {{field.label || field.name}}\n                    {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{field.description ? ': ' + field.description : null}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchDefault>\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"false\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\"\n                [attr.readonly]=\"field.readonly\" [attr.required]=\"field.required\"></anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.minlength\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}}\n                    {{control.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.maxlength\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}}\n                    {{control.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.pattern\">\n                    {{field.label || field.name}}\n                    {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{field.description ? ': ' + field.description : null}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.email\">\n                    {{field.label || field.name}}\n                    {{'must adhere to an email pattern (ex. name@domain.com)' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n</ng-container>",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer2, ChangeDetectorRef])
], SchemaFieldComponent);

var SchemaFormService_1;
let SchemaFormService = SchemaFormService_1 = class SchemaFormService {
    /**
     * @description Recursively parses a raw JSON schema into an array of parsed fields the UI can work with.
     */
    static parseSchema(fieldDefs) {
        const parsedFields = [];
        const fields = fieldDefs.properties;
        for (const propertyName in fields) {
            if (fields[propertyName]) {
                const prop = fields[propertyName];
                // Make all nested fields required if the parent object is required
                if (prop.type === 'object') {
                    prop.required = (fieldDefs.required.includes(propertyName)) ? Object.keys(prop.properties) : [];
                }
                // Handle for table array
                if (prop.type === 'array' && prop.items && prop.items.type === 'object' && prop.items.properties) {
                    prop.parsedArray = SchemaFormService_1.parseSchema(prop.items);
                }
                else if (prop.type === 'array') {
                    let cloneProp = Object.assign({}, prop);
                    cloneProp = Object.assign(cloneProp, prop.items);
                    prop.parsedList = new ParsedField(propertyName, cloneProp);
                }
                else if (prop.type === 'object' && prop.properties) {
                    prop.parsedFields = SchemaFormService_1.parseSchema(prop);
                }
                parsedFields.push(new ParsedField(propertyName, prop, fieldDefs.required, fieldDefs.readOnlyOnUI, fieldDefs.relationships));
            }
        }
        return parsedFields;
    }
    /**
     * @description Parses a tuple of raw JSON and Data object [SchemaDefinition, SchemaData] into a [ParsedField[], FormGroup].
     */
    static parseSchemaPayload(payload) {
        const data = payload[1];
        const fields = SchemaFormService_1.parseSchema(payload[0]);
        const group = SchemaFormService_1.generateFormGroup(fields, data);
        return [fields, group];
    }
    /**
     * @description Recursively generates a formGroup from a ParsedField[], setting any default data provided.
     */
    static generateFormGroup(fields, data) {
        const group = new FormGroup({});
        let i = 1;
        fields.forEach(field => {
            const initialValue = data && data.hasOwnProperty(field.name) ? data[field.name] : field.default;
            // Handle for table array
            if (Array.isArray(field.parsedArray)) {
                const newFormArray = new FormArray([], Validators.compose(SchemaFormService_1.setValidators(field)));
                if (data && data[field.name]) {
                    data[field.name].forEach((dataValue) => {
                        newFormArray.push(SchemaFormService_1.generateFormGroup(field.parsedArray, dataValue));
                    });
                }
                group.addControl(field.name, newFormArray);
            }
            else if (field.parsedList) { // Handle for list array
                const newFormArray = new FormArray([], Validators.compose(SchemaFormService_1.setValidators(field)));
                if (data && data[field.name]) {
                    data[field.name].forEach((dataValue) => {
                        newFormArray.push(new FormControl(dataValue, SchemaFormService_1.setValidators(field.parsedList)));
                    });
                }
                group.addControl(field.name, newFormArray);
            }
            else if (Array.isArray(field.parsedFields)) { // Handle for nested schema
                const nestedGroup = SchemaFormService_1.generateFormGroup(field.parsedFields, initialValue);
                group.addControl(field.name, nestedGroup);
                i++;
            }
            else {
                group.addControl(field.name, new FormControl(initialValue, SchemaFormService_1.setValidators(field)));
            }
        });
        return group;
    }
    /**
     * @description Adds the correct validators to each field pending on its type and whether it is required.
     */
    static setValidators(definition) {
        const validators = [];
        if (definition.required && definition.type === 'boolean') {
            validators.push(AnjunaValidators.requiredCheck);
        }
        else if (definition.required) {
            validators.push(Validators.required);
        }
        if (!isNaN(definition.maximum)) {
            validators.push(Validators.max(definition.maximum));
        }
        if (!isNaN(definition.minimum)) {
            validators.push(Validators.min(definition.minimum));
        }
        if (!isNaN(definition.minLength)) {
            validators.push(Validators.minLength(definition.minLength));
        }
        if (!isNaN(definition.maxLength)) {
            validators.push(Validators.maxLength(definition.maxLength));
        }
        if (definition.format === 'email') {
            validators.push(Validators.email);
        }
        if (definition.pattern) {
            validators.push(Validators.pattern(definition.pattern));
        }
        if (definition.format && definition.format === 'date') {
            validators.push(AnjunaValidators.date);
        }
        if (definition.enforceUnique) {
            validators.push(AnjunaValidators.unique);
        }
        if (!isNaN(definition.minItems) && definition.type === 'array') {
            validators.push(Validators.minLength(definition.minItems));
        }
        return validators;
    }
    /**
     * @description Parses the keys for generating the table renderer
     */
    static parseFieldItemKeys(field) {
        if (field.items && field.items.properties) {
            return Object.keys(field.items.properties);
        }
        return [];
    }
};
SchemaFormService.ɵprov = ɵɵdefineInjectable({ factory: function SchemaFormService_Factory() { return new SchemaFormService(); }, token: SchemaFormService, providedIn: "root" });
SchemaFormService = SchemaFormService_1 = __decorate([
    Injectable({ providedIn: 'root' })
], SchemaFormService);

class SchemaForm {
    constructor(schema, data) {
        this.fields = {};
        this.fieldNames = [];
        const payload = SchemaFormService.parseSchemaPayload([schema, data]);
        this.schema = schema;
        this.data = data;
        this.formGroup = payload[1];
        this.formFields = payload[0].map((field) => {
            field.control = this.formGroup.controls[field.name];
            if (field.parsedFields && field.parsedFields.length > 0) {
                this.setFieldParents(field.parsedFields, field);
                this.flattenFields(field.parsedFields, field.control);
            }
            this.fields[field.name] = field;
            return field;
        });
        this.fieldNames = Object.keys(this.fields).filter(name => (this.fields[name].control !== undefined));
    }
    flattenFields(fields, formGroup) {
        fields.forEach((field) => {
            if (field.parsedFields && field.parsedFields.length > 0) {
                this.flattenFields(field.parsedFields, field.control);
            }
            else {
                if (field.type !== 'array') {
                    field.control = this.findFormControl(field);
                }
                else {
                    field.control = formGroup.controls[field.name];
                }
                this.fields[field.name] = field;
            }
        });
        return fields;
    }
    findFormControl(field) {
        const parentName = field.parent.name;
        let control;
        let parent = field.parent;
        while (parent && !control) {
            const parentControl = parent.control;
            if (parent
                && parentControl
                && parentControl.controls
                && (parentName in parent.control.controls)) {
                control = parent.control.controls[parentName].controls[field.name];
            }
            else {
                parent = parent.parent;
            }
        }
        return control;
    }
    setFieldParents(fields, parent) {
        fields.forEach(field => {
            field.parent = parent;
            if (field.parsedFields && field.parsedFields.length > 0) {
                this.setFieldParents(field.parsedFields, field);
            }
        });
    }
}

let FieldTemplateDirective = class FieldTemplateDirective {
    constructor(_template) {
        this._template = _template;
    }
    get template() {
        return this._template;
    }
};
FieldTemplateDirective.ctorParameters = () => [
    { type: TemplateRef }
];
FieldTemplateDirective = __decorate([
    Directive({
        selector: '[anjFieldTemplate]'
    }),
    __metadata("design:paramtypes", [TemplateRef])
], FieldTemplateDirective);

/* @internal */
let SchemaControl = class SchemaControl extends BaseSchema {
    handleChanges() {
        if (this.data && this.schema) {
            this.initGroup();
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], SchemaControl.prototype, "schema", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SchemaControl.prototype, "data", void 0);
SchemaControl = __decorate([
    Directive()
], SchemaControl);

let SchemaFormComponent = class SchemaFormComponent extends SchemaControl {
    constructor() {
        super();
        this.valueChange = new EventEmitter();
    }
    ngOnChanges() {
        this.handleChanges();
    }
    ngOnDestroy() {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
        }
    }
    /**
     * @description initializes a schema payload imperatively
     */
    initSchemaForm(schemaForm) {
        // Bind them to the component
        this.schemaPayload = schemaForm;
        if (this.schemaPayload) {
            this.groupSubscription = this.schemaPayload.formGroup.valueChanges.subscribe({
                next: (value) => {
                    this.valueChange.emit(value);
                }
            });
        }
    }
    initGroup() {
        this.initSchemaForm(new SchemaForm(this.schema, this.data));
    }
};
__decorate([
    Input(),
    __metadata("design:type", SchemaForm)
], SchemaFormComponent.prototype, "schemaPayload", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], SchemaFormComponent.prototype, "valueChange", void 0);
__decorate([
    ContentChild(FieldTemplateDirective),
    __metadata("design:type", FieldTemplateDirective)
], SchemaFormComponent.prototype, "fieldTemplate", void 0);
SchemaFormComponent = __decorate([
    Component({
        selector: 'anj-schema-form',
        template: "<ng-container *ngIf=\"schemaPayload && fieldTemplate\">\n    <ng-template [ngTemplateOutlet]=\"fieldTemplate?.template\" [ngTemplateOutletContext]=\"{$implicit: schemaPayload.fields, form: schemaPayload.formGroup, fieldNames: schemaPayload.fieldNames}\"></ng-template>\n</ng-container>\n<ng-content select=\"anj-button-group\"></ng-content>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __metadata("design:paramtypes", [])
], SchemaFormComponent);

/* @internal */
let SchemaGroup = class SchemaGroup extends BaseSchema {
    get groupClass() { return this._groupClass || ''; }
    set groupClass(value) {
        this._groupClass = value;
    }
    get fieldClass() { return this._fieldClass || ''; }
    set fieldClass(value) {
        this._fieldClass = value;
    }
    handleChanges() {
        if (this.group) {
            this.initGroup();
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaGroup.prototype, "groupClass", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaGroup.prototype, "fieldClass", null);
__decorate([
    Input(),
    __metadata("design:type", Array)
], SchemaGroup.prototype, "group", void 0);
__decorate([
    HostBinding('attr.fieldClass'),
    __metadata("design:type", String)
], SchemaGroup.prototype, "_fieldClass", void 0);
__decorate([
    HostBinding('attr.groupClass'),
    __metadata("design:type", String)
], SchemaGroup.prototype, "_groupClass", void 0);
SchemaGroup = __decorate([
    Directive()
], SchemaGroup);

let SchemaGroupComponent = class SchemaGroupComponent extends SchemaGroup {
    constructor(changeRef, parentGroup) {
        super();
        this.changeRef = changeRef;
        this.parentGroup = parentGroup;
        // Define the form group when initialized so that it is defined when needed as a view child property
        this.formGroup = new FormGroup({});
        this.valueChange = new EventEmitter();
    }
    get classList() {
        return `anj-schema-group ${this.groupClass}`;
    }
    ngOnChanges() {
        this.handleChanges();
    }
    ngOnDestroy() {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
        }
    }
    /**
     * @description initializes a schema group imperatively
     */
    initFormGroup([fields, group]) {
        // Bind them to the component
        this.parsedFields = fields;
        this.formGroup = group;
        // Integrate to existing parent formGroup, if available
        if (this.parentGroup && this.parentGroup.control) {
            Object.keys(group.controls).forEach(key => {
                this.parentGroup.control.addControl(key, group.controls[key]);
            });
            this.formGroup = this.parentGroup.control;
        }
        this.groupSubscription = this.formGroup.valueChanges.subscribe({
            next: (value) => {
                this.valueChange.emit(value);
            }
        });
    }
    getInputChangeCallback(fieldName) {
        return this.onInputChangeCallbacks ? this.onInputChangeCallbacks[fieldName] : undefined;
    }
    initGroup() {
        this.initFormGroup(SchemaFormService.parseSchemaPayload(this.group));
    }
};
SchemaGroupComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: FormGroupDirective, decorators: [{ type: Optional }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], SchemaGroupComponent.prototype, "onInputChangeCallbacks", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], SchemaGroupComponent.prototype, "valueChange", void 0);
__decorate([
    HostBinding('class'),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], SchemaGroupComponent.prototype, "classList", null);
SchemaGroupComponent = __decorate([
    Component({
        selector: 'anj-schema-group',
        template: "<ng-container *ngIf=\"parsedFields\">\n    <anj-schema-field *ngFor=\"let field of parsedFields; trackBy: trackByFn;\" [field]=\"field\"\n        [control]=\"formGroup.controls[field.name]\" [inputChangeCallback]=\"getInputChangeCallback(field.name)\"\n        class=\"{{fieldClass}}\" [hideLabels]=\"hideLabels\"></anj-schema-field>\n</ng-container>",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [ChangeDetectorRef, FormGroupDirective])
], SchemaGroupComponent);

let SchemaNestedFieldComponent = class SchemaNestedFieldComponent extends SchemaField {
    constructor() {
        super();
    }
    get label() {
        return this._label || '';
    }
    set label(value) {
        this._label = value;
    }
    updateNestedField(newValue) {
        this.control.patchValue(newValue);
    }
};
__decorate([
    Input(),
    __metadata("design:type", AbstractControl)
], SchemaNestedFieldComponent.prototype, "control", void 0);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaNestedFieldComponent.prototype, "label", null);
__decorate([
    HostBinding('class.anj-schema-nested-field'),
    __metadata("design:type", Boolean)
], SchemaNestedFieldComponent.prototype, "isField", void 0);
SchemaNestedFieldComponent = __decorate([
    Component({
        selector: 'anj-schema-nested-field',
        template: "<anj-form-group>\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<div class=\"anj-nested\">\n    <ng-container [formGroup]=\"control\">\n        <anj-schema-field *ngFor=\"let nestedField of field.parsedFields; trackBy: trackByFn;\"\n                          [field]=\"nestedField\"\n                          [control]=\"control['controls'][nestedField.name]\"\n        ></anj-schema-field>\n    </ng-container>\n</div>\n\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        styles: ["anj-schema-nested-field .anj-nested{margin:0 0 1rem 1rem}anj-schema-nested-field .anj-form-group.anj-has-value{margin:0}"]
    }),
    __metadata("design:paramtypes", [])
], SchemaNestedFieldComponent);

let ReadFieldComponent = class ReadFieldComponent extends SchemaField {
    constructor(element, renderer) {
        super();
        this.element = element;
        this.renderer = renderer;
    }
    get fieldItemPropertiesKeys() {
        return SchemaFormService.parseFieldItemKeys(this.field);
    }
    ngAfterViewInit() {
        this.renderer.addClass(this.element.nativeElement, this.field.name.split(' ').join('_'));
    }
    toDisplayValue() {
        let displayValue = this.value ? this.value : '';
        if (Array.isArray(this.value)) {
            displayValue = this.value.join(', ');
        }
        return displayValue;
    }
};
ReadFieldComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], ReadFieldComponent.prototype, "value", void 0);
__decorate([
    HostBinding('class.anj-read-field'),
    __metadata("design:type", Boolean)
], ReadFieldComponent.prototype, "isField", void 0);
ReadFieldComponent = __decorate([
    Component({
        selector: 'anj-read-field',
        template: "<ng-container [ngSwitch]=\"field.toInputType()\">\n    <span *ngIf=\"!hideLabels\" class=\"anj-schema-label\">\n        {{field.label}}\n    </span>\n    <span class=\"anj-schema-display-value\" [ngClass]=\"{'nested': field.toInputType() === 'nested'}\">\n        <ng-template ngSwitchCase=\"date\">\n            {{value | date: 'shortDate'}}\n        </ng-template>\n        <ng-template ngSwitchCase=\"checkbox\">\n            <svg *ngIf=\"value\" xmlns=\"http://www.w3.org/2000/svg\" width=\"18\" height=\"18\" viewBox=\"0 0 18 18\">\n                <path d=\"M6.61 11.89L3.5 8.78 2.44 9.84 6.61 14l8.95-8.95L14.5 4z\"/>\n            </svg>\n        </ng-template>\n        <ng-template ngSwitchCase=\"table\">\n            <anj-table [dataSource]=\"value\">\n                <anj-column *ngFor=\"let key of fieldItemPropertiesKeys;\" [field]=\"key\"></anj-column>\n            </anj-table>\n        </ng-template>\n        <ng-template ngSwitchCase=\"nested\">\n            <anj-schema-read\n                [group]=\"[field.nestedDef, value]\"\n            ></anj-schema-read>\n        </ng-template>\n        <ng-template ngSwitchDefault>\n            {{toDisplayValue()}}\n        </ng-template>\n    </span>\n</ng-container>\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        styles: [".anj-read-field{position:relative}.anj-read-field .nested{margin-left:1rem}.anj-read-field .anj-schema-label{color:var(--anj-neutral);font-size:.75rem;line-height:1;margin-bottom:0;position:relative;top:0}.anj-read-field .anj-schema-display-value{display:block;min-height:1.75rem}"]
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer2])
], ReadFieldComponent);

let SchemaReadComponent = class SchemaReadComponent extends SchemaGroup {
    constructor() {
        super();
    }
    get classList() {
        return `anj-schema-read ${this.groupClass}`;
    }
    ngOnChanges() {
        this.handleChanges();
    }
    initGroup() {
        this.parsedFields = SchemaFormService.parseSchema(this.group[0]);
        this.values = this.group[1] || {};
    }
};
__decorate([
    HostBinding('class'),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], SchemaReadComponent.prototype, "classList", null);
SchemaReadComponent = __decorate([
    Component({
        selector: 'anj-schema-read',
        template: "<ng-container *ngIf=\"parsedFields\">\n    <anj-read-field *ngFor=\"let field of parsedFields\"\n        [field]=\"field\"\n        [value]=\"values[field.name]\"\n        class=\"{{fieldClass}}\"\n        [hideLabels]=\"hideLabels\"\n    ></anj-read-field>\n</ng-container>\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None
    }),
    __metadata("design:paramtypes", [])
], SchemaReadComponent);

/* @internal */
let SchemaArray = class SchemaArray extends SchemaField {
    constructor() {
        super();
    }
    get controlArrayValue() {
        return this.control.value;
    }
    get formArray() {
        return this.control;
    }
    addRow(additionalRows = 1, insertPosition) {
        for (let i = 0; i < additionalRows; i++) {
            const insertIndex = insertPosition !== undefined ? insertPosition : this.controlArrayValue.length;
            this.formArray.insert(insertIndex, this.newItem());
        }
        this.acknowledgeChange();
    }
    removeRow(index) {
        this.formArray.removeAt(index);
        this.acknowledgeChange();
    }
    atMaxItems() {
        return this.field.maxItems && (this.control.length === this.field.maxItems);
    }
    atMinItems() {
        return this.field.minItems && (this.control.length === this.field.minItems);
    }
    showToolTip(event, popper, content) {
        clearTimeout(this.timer);
        this.content = content;
        popper.display(event.target);
    }
    hideToolTip(popper) {
        this.timer = window.setTimeout(() => {
            popper.hide();
        }, 750);
    }
    setArray() {
        // If minItems and data is array shorter than minItems, add more empty rows
        if (this.field.minItems &&
            (Array.isArray(this.controlArrayValue) && this.controlArrayValue.length < this.field.minItems)) {
            const additionalRows = this.field.minItems - this.controlArrayValue.length;
            this.addRow(additionalRows);
            // If Data is not an array, add minimum rows (minItems or 1)
        }
        else if (!Array.isArray(this.controlArrayValue)) {
            const additionalRows = (this.field.minItems > 0) ? this.field.minItems : 1;
            this.addRow(additionalRows);
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormArray)
], SchemaArray.prototype, "control", void 0);
SchemaArray = __decorate([
    Directive(),
    __metadata("design:paramtypes", [])
], SchemaArray);

let SchemaTableComponent = class SchemaTableComponent extends SchemaArray {
    constructor(changeRef) {
        super();
        this.changeRef = changeRef;
    }
    get label() {
        return this._label || '';
    }
    set label(value) {
        this._label = value;
    }
    // Listen to rows added on anj-table, modify the NG Control respectively
    _onRowAdd(event) {
        this.addRow(1, event.detail.index);
    }
    // Listen to rows removed on anj-table, modify the NG Control respectively
    _onRowRemove(event) {
        this.removeRow(event.detail.index);
    }
    get fieldItemPropertiesKeys() {
        return SchemaFormService.parseFieldItemKeys(this.field);
    }
    ngOnInit() {
        this.localControlValue = this.controlArrayValue;
        this.setArray();
    }
    ngOnChanges(changes) {
        this.acknowledgeChange();
    }
    getDataType(def) {
        // TODO: we probably need to handle at least numbers too
        if (def.type === 'boolean') {
            return 'boolean';
        }
    }
    findIndex(fields, c) {
        return fields.findIndex(t => t.name === c.field);
    }
    newItem() {
        return SchemaFormService.generateFormGroup(this.field.parsedArray, {});
    }
    acknowledgeChange() {
        this.localControlValue = this.controlArrayValue.slice(0);
        this.changeRef.markForCheck();
    }
};
SchemaTableComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaTableComponent.prototype, "label", null);
__decorate([
    HostBinding('class.anj-schema-table'),
    __metadata("design:type", Boolean)
], SchemaTableComponent.prototype, "isField", void 0);
__decorate([
    HostListener('anjCreateStart', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [CustomEvent]),
    __metadata("design:returntype", void 0)
], SchemaTableComponent.prototype, "_onRowAdd", null);
__decorate([
    HostListener('anjDelete', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [CustomEvent]),
    __metadata("design:returntype", void 0)
], SchemaTableComponent.prototype, "_onRowRemove", null);
SchemaTableComponent = __decorate([
    Component({
        selector: 'anj-schema-table',
        template: "<anj-form-group class=\"anj-schema-table-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<anj-table [dataSource]=\"localControlValue\" column-width=\"auto-fix\" height=\"400px\">\n    <anj-editing\n        mode=\"batch\"\n        creating=\"true\"\n        deleting=\"true\"\n        saving=\"false\"\n        [deleteDisabled]=\"atMinItems()\"\n        [createDisabled]=\"atMaxItems()\"\n    ></anj-editing>\n    <anj-column *ngFor=\"let key of fieldItemPropertiesKeys; let index = index;\"\n                [dataType]=\"getDataType(field.parsedArray[index])\"\n                [field]=\"key\"\n                sortable=\"false\"\n                edit-template=\"controlTemplate\"\n    ></anj-column>\n    <ng-container *anjTemplate=\"let row of 'controlTemplate'; let i = index; let c = column;\">\n        <anj-schema-field\n            *ngIf=\"control.controls[i]\"\n            [field]=\"field.parsedArray[findIndex(field.parsedArray, c)]\"\n            [control]=\"control.controls[i]['controls'][c.field]\"\n            [hideLabels]=\"true\">\n        </anj-schema-field>\n    </ng-container>\n</anj-table>\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        styles: ["anj-schema-table .anj-schema-table-label{margin-top:1rem}anj-schema-table anj-schema-field{display:block}"]
    }),
    __metadata("design:paramtypes", [ChangeDetectorRef])
], SchemaTableComponent);

let SchemaContextComponent = class SchemaContextComponent extends BaseSchema {
    constructor() {
        super();
        this.valueChange = new EventEmitter();
    }
    get groupClass() { return this._groupClass || ''; }
    set groupClass(value) {
        this._groupClass = value;
    }
    get fieldClass() { return this._fieldClass || ''; }
    set fieldClass(value) {
        this._fieldClass = value;
    }
    get mode() { return this._mode || 'view'; }
    set mode(value) {
        this._mode = value;
    }
    get classList() {
        return `anj-schema-context anj-schema-context-${this.context}`;
    }
    updateApp(event) {
        this.valueChange.emit(event);
    }
    ngOnChanges() {
        this.schemaPayload = this.combinePayload();
    }
    combinePayload() {
        // Create Mode
        if (this.mode === 'create' && this.schema) {
            return [this.schema, null];
            // View Mode
        }
        else if (this.mode === 'view' && this.schema && !this.data) {
            return [this.schema, {}];
            // Update Mode
        }
        else if (this.schema && this.data) {
            return [this.schema, this.data];
        }
        return null;
    }
};
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaContextComponent.prototype, "groupClass", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaContextComponent.prototype, "fieldClass", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], SchemaContextComponent.prototype, "mode", null);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SchemaContextComponent.prototype, "schema", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SchemaContextComponent.prototype, "data", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SchemaContextComponent.prototype, "context", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], SchemaContextComponent.prototype, "valueChange", void 0);
__decorate([
    ViewChild(SchemaGroupComponent),
    __metadata("design:type", SchemaGroupComponent)
], SchemaContextComponent.prototype, "schemaGroup", void 0);
__decorate([
    HostBinding('class'),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], SchemaContextComponent.prototype, "classList", null);
__decorate([
    HostBinding('attr.fieldClass'),
    __metadata("design:type", String)
], SchemaContextComponent.prototype, "_fieldClass", void 0);
__decorate([
    HostBinding('attr.groupClass'),
    __metadata("design:type", String)
], SchemaContextComponent.prototype, "_groupClass", void 0);
__decorate([
    HostBinding('attr.mode'),
    __metadata("design:type", String)
], SchemaContextComponent.prototype, "_mode", void 0);
SchemaContextComponent = __decorate([
    Component({
        selector: 'anj-schema-context, anj-custom-fields',
        template: "<ng-container *ngIf=\"mode === 'view'; else formFields\">\n    <anj-schema-read\n        [group]=\"schemaPayload\"\n        [groupClass]=\"groupClass\"\n        [fieldClass]=\"fieldClass\"\n        [hideLabels]=\"hideLabels\"\n    ></anj-schema-read>\n</ng-container>\n<ng-template #formFields>\n    <anj-schema-group\n        [group]=\"schemaPayload\"\n        [groupClass]=\"groupClass\"\n        [fieldClass]=\"fieldClass\"\n        [hideLabels]=\"hideLabels\"\n        (valueChange)=\"updateApp($event)\"\n    ></anj-schema-group>\n</ng-template>\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None
    }),
    __metadata("design:paramtypes", [])
], SchemaContextComponent);

const translations = {
    en: {
        'is required.': 'is required.',
        'is not a valid date.': 'is not a valid date.',
        'Please select an option.': 'Please select an option.',
        'must be at least': 'must be at least',
        'must not be greater than': 'must not be greater than',
        'must not have duplicate values.': 'must not have duplicate values.',
        'must have at least': 'must have at least',
        'entries.': 'entries.',
        'characters.': 'characters.',
        'must adhere to a specific pattern': 'must adhere to a specific pattern',
        'must adhere to an email pattern (ex. name@domain.com)': 'must adhere to an email pattern (ex. name@domain.com)',
        'Add an item below': 'Add an item below',
        'Remove this item': 'Remove this item',
        'Add a row below': 'Add a row below',
        'Remove this row': 'Remove this row',
    },
    fr: {
        'is required.': 'est obligatoire.',
        'is not a valid date.': 'n’est pas une date valide.',
        'Please select an option.': 'Sélectionnez une option.',
        'must be at least': 'doit être au moins',
        'must not be greater than': 'ne doit pas dépasser',
        'must not have duplicate values.': 'ne doit pas inclure de valeurs dupliquées.',
        'must have at least': 'doit inclure au moins',
        'entries.': 'entrées.',
        'characters.': 'caractères.',
        'must adhere to a specific pattern': 'doit correspondre à un format spécifique',
        'must adhere to an email pattern (ex. name@domain.com)': 'doit correspondre à un format d’e-mail (par ex., nom@domaine.com)',
        'Add an item below': 'Ajouter un élément ci-dessous',
        'Remove this item': 'Supprimer cet élément',
        'Add a row below': 'Ajouter une ligne ci-dessous',
        'Remove this row': 'Supprimer cette ligne',
    },
    de: {
        'is required.': 'ist erforderlich.',
        'is not a valid date.': 'ist kein gültiges Datum.',
        'Please select an option.': 'Wählen Sie eine Option aus.',
        'must be at least': 'muss mindestens folgenden Wert haben:',
        'must not be greater than': 'darf nicht größer als folgender Wert sein:',
        'must not have duplicate values.': 'darf keine duplizierten Werte aufweisen.',
        'must have at least': 'muss mindestens',
        'entries.': 'Einträge aufweisen.',
        'characters.': 'Zeichen lang sein.',
        'must adhere to a specific pattern': 'muss ein bestimmtes Muster befolgen',
        'must adhere to an email pattern (ex. name@domain.com)': 'muss ein bestimmtes E-Mail-Muster befolgen (Bsp.: name@domain.com)',
        'Add an item below': 'Element unterhalb hinzufügen',
        'Remove this item': 'Dieses Element entfernen',
        'Add a row below': 'Zeile unterhalb hinzufügen',
        'Remove this row': 'Diese Zeile entfernen',
    },
    ja: {
        'is required.': 'は必須です。',
        'is not a valid date.': 'は有効な日付ではありません。',
        'Please select an option.': 'オプションを選択してください。',
        'must be at least': 'の最小長さは',
        'must not be greater than': 'の最大長さは',
        'must not have duplicate values.': 'の値は重複させることはできません。',
        'must have at least': 'には少なくとも',
        'entries.': '個のエントリが必要です。',
        'characters.': '文字にする必要があります。',
        'must adhere to a specific pattern': 'は指定のパターン',
        'must adhere to an email pattern (ex. name@domain.com)': 'は電子メールのパターン (name@domain.com など) に従う必要があります',
        'Add an item below': '以下のアイテムを追加',
        'Remove this item': 'このアイテムを削除',
        'Add a row below': '以下の行を追加',
        'Remove this row': 'この行を削除',
    },
    es: {
        'is required.': 'es obligatoria.',
        'is not a valid date.': 'no es una fecha válida',
        'Please select an option': 'Por favor, seleccione una opción',
        'must be at least': 'debe ser al menos',
        'must not be greater than': 'no debe ser mayor que',
        'must not have duplicate values': 'no debe contener valores duplicados',
        'must have at least': 'debe tener al menos',
        'entries.': 'entradas.',
        'characters.': 'caracteres.',
        'must adhere to a specific pattern': 'debe ajustarse a un patrón específico',
        'must adhere to an email pattern (ex. name@domain.com)': 'debe adherirse a un patrón de correo electrónico (por ejemplo, nombre@dominio.com)',
        'Add an item below': 'Agregar un elemento abajo',
        'Remove this item': 'Eliminar este elemento',
        'Add a row below': 'Agregar una fila abajo',
        'Remove this row': 'Eliminar esta fila'
    }
};

/** @internal */
let SchemaTranslationPipe = class SchemaTranslationPipe extends BaseTranslationPipe {
    constructor() {
        super(...arguments);
        this.translations = translations;
    }
};
SchemaTranslationPipe = __decorate([
    Pipe({
        name: 'anjSchemaTranslate'
    })
], SchemaTranslationPipe);

let SchemaTierComponent = class SchemaTierComponent extends SchemaTableComponent {
    constructor(changeRef, el) {
        super(changeRef);
        this.el = el;
        this.uom = { value: 1 };
    }
    ngOnInit() {
        super.ngOnInit();
        console.log(this.field, '<<');
        this.uoms = (this.field.uoms)
            ? this.field.uoms.map(f => ({ name: f.name, value: Number.parseFloat(f.value) }))
            : undefined;
        const initialSchemaUOM = this.uoms.find(u => u.name === this.field.uom);
        this.uom = initialSchemaUOM || { value: 1 };
        this.refreshTiers();
    }
    _onRowAdd(event) {
        const length = this.control.length;
        this.addRow(1, length);
    }
    // Passthrough to manage the readonly features of the first column (sparing the first row)
    getField(column, rowIndex) {
        const columnIndex = this.findIndex(this.field.parsedArray, column);
        const field = this.field.parsedArray[columnIndex];
        field['readonly'] = !!(columnIndex === 0 && rowIndex > 0);
        return field;
    }
    tierToChange(index) {
        const nextRow = this.control.at(index + 1);
        const currentRowTierTo = this.control.at(index).get('tierTo').value;
        if (nextRow) {
            nextRow.patchValue({ tierFrom: currentRowTierTo + this.uom.value });
        }
    }
    setUom({ detail }) {
        this.uom = detail.value;
        const domEvent = new CustomEvent('uomChange', {
            detail: this.uom,
            bubbles: true
        });
        this.el.nativeElement.dispatchEvent(domEvent);
        this.refreshTiers();
    }
    newItem() {
        const group = SchemaFormService.generateFormGroup(this.field.parsedArray, {});
        const previousTier = this.control.at(this.control.length - 1);
        const previousTierTo = previousTier ? Number.parseFloat(previousTier.get('tierTo').value) : 0;
        group.patchValue({ tierFrom: previousTierTo + this.uom.value });
        return group;
    }
    refreshTiers() {
        const controls = this.control.controls;
        for (let i = 0; i < controls.length; i++) {
            if (i > 0) {
                const previousTier = controls[i - 1].value;
                const endingUnit = Number.parseFloat(previousTier.tierTo);
                controls[i].patchValue({ tierFrom: endingUnit + this.uom.value });
            }
        }
        this.acknowledgeChange();
    }
};
SchemaTierComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ElementRef }
];
SchemaTierComponent = __decorate([
    Component({
        selector: 'anj-schema-tier',
        template: "<anj-form-group class=\"anj-schema-table-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<anj-table [dataSource]=\"localControlValue\" column-width=\"auto-fix\" height=\"400px\">\n    <anj-editing\n        mode=\"batch\"\n        creating=\"true\"\n        deleting=\"true\"\n        saving=\"false\"\n        [deleteDisabled]=\"atMinItems()\"\n        [createDisabled]=\"atMaxItems()\"\n    ></anj-editing>\n    <anj-column *ngFor=\"let key of fieldItemPropertiesKeys; let index = index;\"\n                [dataType]=\"getDataType(field.parsedArray[index])\"\n                [field]=\"key\"\n                sortable=\"false\"\n                edit-template=\"controlTemplate\"\n    ></anj-column>\n    <ng-container *anjTemplate=\"let row of 'controlTemplate'; let i = index; let c = column;\">\n        <anj-schema-field\n            *ngIf=\"control.controls[i]\"\n            [field]=\"getField(c, i)\"\n            [control]=\"control.controls[i]['controls'][c.field]\"\n            [index]=\"i\"\n            [hideLabels]=\"true\"\n            (tierToChange)=\"tierToChange($event)\">\n        </anj-schema-field>\n    </ng-container>\n</anj-table>\n<anj-form-group *ngIf=\"uoms\">\n    <anj-label>Unit of Measurement</anj-label>\n    <anj-select (anjChange)=\"setUom($event)\" [value]=\"uom\">\n        <anj-option *ngFor=\"let u of uoms\" [value]=\"u\">{{u.name}}</anj-option>\n    </anj-select>\n</anj-form-group>\n",
        styles: ["anj-schema-table .anj-schema-table-label{margin-top:1rem}anj-schema-table anj-schema-field{display:block}"]
    }),
    __metadata("design:paramtypes", [ChangeDetectorRef, ElementRef])
], SchemaTierComponent);

let AnjunaMetaUIModule = class AnjunaMetaUIModule {
};
AnjunaMetaUIModule = __decorate([
    NgModule({
        declarations: [
            FieldTemplateDirective,
            SchemaFieldComponent,
            SchemaFormComponent,
            SchemaGroupComponent,
            SchemaNestedFieldComponent,
            SchemaTranslationPipe,
            SchemaReadComponent,
            SchemaTableComponent,
            ReadFieldComponent,
            SchemaContextComponent,
            SchemaTierComponent
        ],
        imports: [
            CommonModule,
            ReactiveFormsModule,
            FormsModule,
            AnjunaCoreModule
        ],
        exports: [
            FieldTemplateDirective,
            SchemaFieldComponent,
            SchemaFormComponent,
            SchemaGroupComponent,
            SchemaNestedFieldComponent,
            SchemaReadComponent,
            SchemaTableComponent,
            ReadFieldComponent,
            SchemaContextComponent
        ],
        schemas: [CUSTOM_ELEMENTS_SCHEMA]
    })
], AnjunaMetaUIModule);

/* Components */

/**
 * Generated bundle index. Do not edit.
 */

export { AnjunaMetaUIModule, FieldTemplateDirective, ParsedField, ReadFieldComponent, SchemaContextComponent, SchemaFieldComponent, SchemaForm, SchemaFormComponent, SchemaFormService, SchemaGroupComponent, SchemaNestedFieldComponent, SchemaReadComponent, SchemaTableComponent, SchemaField as ɵa, BaseSchema as ɵb, SchemaControl as ɵc, SchemaGroup as ɵd, SchemaArray as ɵe, SchemaTranslationPipe as ɵf, SchemaTierComponent as ɵg };
//# sourceMappingURL=anjuna-angular-meta-ui.js.map
