import { __decorate, __metadata, __extends, __awaiter, __generator, __read, __param } from 'tslib';
import { Input, HostBinding, Directive, EventEmitter, ElementRef, Renderer2, ChangeDetectorRef, Output, HostListener, Component, ChangeDetectionStrategy, ViewEncapsulation, ɵɵdefineInjectable, Injectable, TemplateRef, ContentChild, Optional, ViewChild, Pipe, NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { AbstractControl, FormGroup, FormArray, Validators, FormControl, FormGroupDirective, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { evalToBool } from '@anjuna/core';
import { AnjunaValidators, BaseTranslationPipe, AnjunaCoreModule } from '@anjuna/angular-core';
import { CommonModule } from '@angular/common';

/* @internal */
var BaseSchema = /** @class */ (function () {
    function BaseSchema() {
    }
    Object.defineProperty(BaseSchema.prototype, "hideLabels", {
        get: function () {
            return this._hideLabels;
        },
        set: function (value) {
            if (typeof value === 'string') {
                this._hideLabels = evalToBool(value);
            }
            else {
                this._hideLabels = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    BaseSchema.prototype.trackByFn = function (index) {
        return index;
    };
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], BaseSchema.prototype, "hideLabels", null);
    __decorate([
        HostBinding('attr.hideLabel'),
        __metadata("design:type", Boolean)
    ], BaseSchema.prototype, "_hideLabels", void 0);
    BaseSchema = __decorate([
        Directive()
    ], BaseSchema);
    return BaseSchema;
}());

var ParsedField = /** @class */ (function () {
    function ParsedField(name, fieldDef, requiredFields, readonlyFields, relationships) {
        if (requiredFields === void 0) { requiredFields = []; }
        if (readonlyFields === void 0) { readonlyFields = []; }
        if (relationships === void 0) { relationships = []; }
        this.default = fieldDef.default;
        this.description = fieldDef.description;
        this.enforceUnique = fieldDef.uniqueItems;
        this.format = fieldDef.format;
        this.items = fieldDef.items;
        this.label = fieldDef.title || fieldDef.label || name;
        this.maximum = fieldDef.maximum;
        this.maxItems = fieldDef.maxItems;
        this.maxLength = fieldDef.maxLength;
        this.minimum = fieldDef.minimum;
        this.minItems = fieldDef.minItems;
        this.minLength = fieldDef.minLength;
        this.name = name;
        this.nestedDef = fieldDef;
        this.options = fieldDef.enum;
        this.pattern = fieldDef.pattern;
        this.parsedArray = fieldDef.parsedArray;
        this.parsedList = fieldDef.parsedList;
        this.parsedFields = fieldDef.parsedFields;
        this.properties = fieldDef.properties;
        this.readonly = fieldDef.readonly || (readonlyFields ? readonlyFields.includes(name) : false);
        this.required = requiredFields ? requiredFields.includes(name) : false;
        this.subtype = fieldDef.subtype;
        this.type = fieldDef.type;
        this.uoms = fieldDef.UOMs;
        this.uom = fieldDef.UOM;
        this.relationships = relationships;
    }
    ParsedField.prototype.toInputType = function () {
        var _this = this;
        if (this.format === 'date') {
            return 'date';
        }
        else if (this.options) {
            return 'select';
        }
        else if (this.type === 'integer' || this.type === 'number') {
            return 'number';
        }
        else if (this.type === 'boolean') {
            return 'checkbox';
        }
        else if (this.type === 'array' && this.subtype === 'tier') {
            return 'tier';
        }
        else if (this.type === 'array' && this.items && this.items.type === 'object') {
            return 'table';
        }
        else if (this.type === 'array' && this.items) {
            return 'list';
        }
        else if (this.type === 'object' && this.properties) {
            return 'nested';
        }
        else if (this.type === 'string' && this.subtype === 'area') {
            return 'area';
        }
        else if (this.format === 'uuid' &&
            this.relationships.some(function (r) { return Object.keys(r.fields).includes(_this.name); })) {
            return 'relationship';
        }
        return 'text';
    };
    return ParsedField;
}());

/* @internal */
var SchemaField = /** @class */ (function (_super) {
    __extends(SchemaField, _super);
    function SchemaField() {
        var _this = _super.call(this) || this;
        _this.isField = true;
        return _this;
    }
    __decorate([
        Input(),
        __metadata("design:type", ParsedField)
    ], SchemaField.prototype, "field", void 0);
    SchemaField = __decorate([
        Directive(),
        __metadata("design:paramtypes", [])
    ], SchemaField);
    return SchemaField;
}(BaseSchema));

var SchemaFieldComponent = /** @class */ (function (_super) {
    __extends(SchemaFieldComponent, _super);
    function SchemaFieldComponent(element, renderer, changeRef) {
        var _this = _super.call(this) || this;
        _this.element = element;
        _this.renderer = renderer;
        _this.changeRef = changeRef;
        _this.tierToChange = new EventEmitter();
        return _this;
    }
    SchemaFieldComponent.prototype._onFieldChange = function (event) {
        if (this.field.name === 'tierTo') {
            this.tierToChange.emit(this.index);
        }
    };
    SchemaFieldComponent.prototype.ngAfterViewInit = function () {
        this.renderer.addClass(this.element.nativeElement, this.field.name.split(' ').join('_'));
    };
    SchemaFieldComponent.prototype.handleOnChangeCallback = function (_a) {
        var detail = _a.detail;
        return __awaiter(this, void 0, void 0, function () {
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (!this.inputChangeCallback) return [3 /*break*/, 2];
                        _b = this;
                        return [4 /*yield*/, this.inputChangeCallback(detail)];
                    case 1:
                        _b.typeaheadOptions = _c.sent();
                        this.changeRef.markForCheck();
                        _c.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    SchemaFieldComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", AbstractControl)
    ], SchemaFieldComponent.prototype, "control", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], SchemaFieldComponent.prototype, "index", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], SchemaFieldComponent.prototype, "inputChangeCallback", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SchemaFieldComponent.prototype, "tierToChange", void 0);
    __decorate([
        HostBinding("class.anj-schema-field"),
        __metadata("design:type", Boolean)
    ], SchemaFieldComponent.prototype, "isField", void 0);
    __decorate([
        HostListener('anjChange', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [CustomEvent]),
        __metadata("design:returntype", void 0)
    ], SchemaFieldComponent.prototype, "_onFieldChange", null);
    SchemaFieldComponent = __decorate([
        Component({
            selector: 'anj-schema-field',
            template: "<ng-container [ngSwitch]=\"field.toInputType()\">\n    <ng-template ngSwitchCase=\"relationship\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-type-ahead (anjQuery)=\"handleOnChangeCallback($event)\" [formControl]=\"control\" display=\"label\">\n                <anj-option *ngFor=\"let option of typeaheadOptions\" [value]=\"option\">\n                    {{option.label}}\n                </anj-option>\n            </anj-type-ahead>\n        </anj-form-group>\n        <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n    </ng-template>\n    <ng-template ngSwitchCase=\"date\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\" [min]=\"field.minimum\"\n                [max]=\"field.maximum\" [attr.readonly]=\"field.readonly\" [attr.required]=\"field.required\">\n                <anj-date-picker-toggle></anj-date-picker-toggle>\n                <anj-date-picker></anj-date-picker>\n            </anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.date?.valid === false\">\n                    {{field.label || field.name}} {{'is not a valid date.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"select\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-select placeholder=\"{{hideLabels ? field.name : null}}\" [formControl]=\"control\"\n                [attr.required]=\"field.required\">\n                <anj-option *ngFor=\"let option of field.options\" [value]=\"option\">{{option}}</anj-option>\n                <ng-template [ngIf]=\"control.errors\">\n                    <anj-form-error>{{'Please select an option.' | anjSchemaTranslate}}</anj-form-error>\n                </ng-template>\n            </anj-select>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"number\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" step=\"1\" type=\"number\" placeholder=\"{{hideLabels ? field.name : null}}\"\n                [min]=\"field.minimum\" [max]=\"field.maximum\" [attr.readonly]=\"field.readonly\"\n                [attr.required]=\"field.required\">\n                <anj-number-stepper></anj-number-stepper>\n            </anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.min\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}}\n                    {{control.errors.min.min}}.\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.max\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}}\n                    {{control.errors.max.max}}.\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"checkbox\">\n        <anj-check-group>\n            <anj-checkbox [formControl]=\"control\">{{field.label}}</anj-checkbox>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n        </anj-check-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"list\">\n        <anj-form-group>\n            <anj-list [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\"\n                [attr.required]=\"field.required\"></anj-list>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.unique\">\n                    {{field.label || field.name}} {{'must not have duplicate values.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.minlength\">\n                    {{field.label || field.name}} {{'must have at least' | anjSchemaTranslate}}\n                    {{control.errors['minlength'].requiredLength}} {{'entries.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"nested\">\n        <anj-schema-nested-field [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\">\n        </anj-schema-nested-field>\n    </ng-template>\n    <ng-template ngSwitchCase=\"table\">\n        <anj-schema-table [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\">\n        </anj-schema-table>\n    </ng-template>\n    <ng-template ngSwitchCase=\"tier\">\n        <anj-schema-tier [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\">\n        </anj-schema-tier>\n    </ng-template>\n    <ng-template ngSwitchCase=\"area\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"!field.required\">{{field.label}}</anj-label>\n            <anj-textarea [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\"\n                [attr.readonly]=\"field.readonly\" [attr.required]=\"field.required\"></anj-textarea>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.minlength\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}}\n                    {{control.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.maxlength\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}}\n                    {{control.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.pattern\">\n                    {{field.label || field.name}}\n                    {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{field.description ? ': ' + field.description : null}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchDefault>\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\" [optional]=\"false\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\"\n                [attr.readonly]=\"field.readonly\" [attr.required]=\"field.required\"></anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.minlength\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}}\n                    {{control.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.maxlength\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}}\n                    {{control.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.pattern\">\n                    {{field.label || field.name}}\n                    {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{field.description ? ': ' + field.description : null}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.email\">\n                    {{field.label || field.name}}\n                    {{'must adhere to an email pattern (ex. name@domain.com)' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n</ng-container>",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None
        }),
        __metadata("design:paramtypes", [ElementRef, Renderer2, ChangeDetectorRef])
    ], SchemaFieldComponent);
    return SchemaFieldComponent;
}(SchemaField));

var SchemaFormService = /** @class */ (function () {
    function SchemaFormService() {
    }
    SchemaFormService_1 = SchemaFormService;
    /**
     * @description Recursively parses a raw JSON schema into an array of parsed fields the UI can work with.
     */
    SchemaFormService.parseSchema = function (fieldDefs) {
        var parsedFields = [];
        var fields = fieldDefs.properties;
        for (var propertyName in fields) {
            if (fields[propertyName]) {
                var prop = fields[propertyName];
                // Make all nested fields required if the parent object is required
                if (prop.type === 'object') {
                    prop.required = (fieldDefs.required.includes(propertyName)) ? Object.keys(prop.properties) : [];
                }
                // Handle for table array
                if (prop.type === 'array' && prop.items && prop.items.type === 'object' && prop.items.properties) {
                    prop.parsedArray = SchemaFormService_1.parseSchema(prop.items);
                }
                else if (prop.type === 'array') {
                    var cloneProp = Object.assign({}, prop);
                    cloneProp = Object.assign(cloneProp, prop.items);
                    prop.parsedList = new ParsedField(propertyName, cloneProp);
                }
                else if (prop.type === 'object' && prop.properties) {
                    prop.parsedFields = SchemaFormService_1.parseSchema(prop);
                }
                parsedFields.push(new ParsedField(propertyName, prop, fieldDefs.required, fieldDefs.readOnlyOnUI, fieldDefs.relationships));
            }
        }
        return parsedFields;
    };
    /**
     * @description Parses a tuple of raw JSON and Data object [SchemaDefinition, SchemaData] into a [ParsedField[], FormGroup].
     */
    SchemaFormService.parseSchemaPayload = function (payload) {
        var data = payload[1];
        var fields = SchemaFormService_1.parseSchema(payload[0]);
        var group = SchemaFormService_1.generateFormGroup(fields, data);
        return [fields, group];
    };
    /**
     * @description Recursively generates a formGroup from a ParsedField[], setting any default data provided.
     */
    SchemaFormService.generateFormGroup = function (fields, data) {
        var group = new FormGroup({});
        var i = 1;
        fields.forEach(function (field) {
            var initialValue = data && data.hasOwnProperty(field.name) ? data[field.name] : field.default;
            // Handle for table array
            if (Array.isArray(field.parsedArray)) {
                var newFormArray_1 = new FormArray([], Validators.compose(SchemaFormService_1.setValidators(field)));
                if (data && data[field.name]) {
                    data[field.name].forEach(function (dataValue) {
                        newFormArray_1.push(SchemaFormService_1.generateFormGroup(field.parsedArray, dataValue));
                    });
                }
                group.addControl(field.name, newFormArray_1);
            }
            else if (field.parsedList) { // Handle for list array
                var newFormArray_2 = new FormArray([], Validators.compose(SchemaFormService_1.setValidators(field)));
                if (data && data[field.name]) {
                    data[field.name].forEach(function (dataValue) {
                        newFormArray_2.push(new FormControl(dataValue, SchemaFormService_1.setValidators(field.parsedList)));
                    });
                }
                group.addControl(field.name, newFormArray_2);
            }
            else if (Array.isArray(field.parsedFields)) { // Handle for nested schema
                var nestedGroup = SchemaFormService_1.generateFormGroup(field.parsedFields, initialValue);
                group.addControl(field.name, nestedGroup);
                i++;
            }
            else {
                group.addControl(field.name, new FormControl(initialValue, SchemaFormService_1.setValidators(field)));
            }
        });
        return group;
    };
    /**
     * @description Adds the correct validators to each field pending on its type and whether it is required.
     */
    SchemaFormService.setValidators = function (definition) {
        var validators = [];
        if (definition.required && definition.type === 'boolean') {
            validators.push(AnjunaValidators.requiredCheck);
        }
        else if (definition.required) {
            validators.push(Validators.required);
        }
        if (!isNaN(definition.maximum)) {
            validators.push(Validators.max(definition.maximum));
        }
        if (!isNaN(definition.minimum)) {
            validators.push(Validators.min(definition.minimum));
        }
        if (!isNaN(definition.minLength)) {
            validators.push(Validators.minLength(definition.minLength));
        }
        if (!isNaN(definition.maxLength)) {
            validators.push(Validators.maxLength(definition.maxLength));
        }
        if (definition.format === 'email') {
            validators.push(Validators.email);
        }
        if (definition.pattern) {
            validators.push(Validators.pattern(definition.pattern));
        }
        if (definition.format && definition.format === 'date') {
            validators.push(AnjunaValidators.date);
        }
        if (definition.enforceUnique) {
            validators.push(AnjunaValidators.unique);
        }
        if (!isNaN(definition.minItems) && definition.type === 'array') {
            validators.push(Validators.minLength(definition.minItems));
        }
        return validators;
    };
    /**
     * @description Parses the keys for generating the table renderer
     */
    SchemaFormService.parseFieldItemKeys = function (field) {
        if (field.items && field.items.properties) {
            return Object.keys(field.items.properties);
        }
        return [];
    };
    var SchemaFormService_1;
    SchemaFormService.ɵprov = ɵɵdefineInjectable({ factory: function SchemaFormService_Factory() { return new SchemaFormService(); }, token: SchemaFormService, providedIn: "root" });
    SchemaFormService = SchemaFormService_1 = __decorate([
        Injectable({ providedIn: 'root' })
    ], SchemaFormService);
    return SchemaFormService;
}());

var SchemaForm = /** @class */ (function () {
    function SchemaForm(schema, data) {
        var _this = this;
        this.fields = {};
        this.fieldNames = [];
        var payload = SchemaFormService.parseSchemaPayload([schema, data]);
        this.schema = schema;
        this.data = data;
        this.formGroup = payload[1];
        this.formFields = payload[0].map(function (field) {
            field.control = _this.formGroup.controls[field.name];
            if (field.parsedFields && field.parsedFields.length > 0) {
                _this.setFieldParents(field.parsedFields, field);
                _this.flattenFields(field.parsedFields, field.control);
            }
            _this.fields[field.name] = field;
            return field;
        });
        this.fieldNames = Object.keys(this.fields).filter(function (name) { return (_this.fields[name].control !== undefined); });
    }
    SchemaForm.prototype.flattenFields = function (fields, formGroup) {
        var _this = this;
        fields.forEach(function (field) {
            if (field.parsedFields && field.parsedFields.length > 0) {
                _this.flattenFields(field.parsedFields, field.control);
            }
            else {
                if (field.type !== 'array') {
                    field.control = _this.findFormControl(field);
                }
                else {
                    field.control = formGroup.controls[field.name];
                }
                _this.fields[field.name] = field;
            }
        });
        return fields;
    };
    SchemaForm.prototype.findFormControl = function (field) {
        var parentName = field.parent.name;
        var control;
        var parent = field.parent;
        while (parent && !control) {
            var parentControl = parent.control;
            if (parent
                && parentControl
                && parentControl.controls
                && (parentName in parent.control.controls)) {
                control = parent.control.controls[parentName].controls[field.name];
            }
            else {
                parent = parent.parent;
            }
        }
        return control;
    };
    SchemaForm.prototype.setFieldParents = function (fields, parent) {
        var _this = this;
        fields.forEach(function (field) {
            field.parent = parent;
            if (field.parsedFields && field.parsedFields.length > 0) {
                _this.setFieldParents(field.parsedFields, field);
            }
        });
    };
    return SchemaForm;
}());

var FieldTemplateDirective = /** @class */ (function () {
    function FieldTemplateDirective(_template) {
        this._template = _template;
    }
    Object.defineProperty(FieldTemplateDirective.prototype, "template", {
        get: function () {
            return this._template;
        },
        enumerable: true,
        configurable: true
    });
    FieldTemplateDirective.ctorParameters = function () { return [
        { type: TemplateRef }
    ]; };
    FieldTemplateDirective = __decorate([
        Directive({
            selector: '[anjFieldTemplate]'
        }),
        __metadata("design:paramtypes", [TemplateRef])
    ], FieldTemplateDirective);
    return FieldTemplateDirective;
}());

/* @internal */
var SchemaControl = /** @class */ (function (_super) {
    __extends(SchemaControl, _super);
    function SchemaControl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SchemaControl.prototype.handleChanges = function () {
        if (this.data && this.schema) {
            this.initGroup();
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SchemaControl.prototype, "schema", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SchemaControl.prototype, "data", void 0);
    SchemaControl = __decorate([
        Directive()
    ], SchemaControl);
    return SchemaControl;
}(BaseSchema));

var SchemaFormComponent = /** @class */ (function (_super) {
    __extends(SchemaFormComponent, _super);
    function SchemaFormComponent() {
        var _this = _super.call(this) || this;
        _this.valueChange = new EventEmitter();
        return _this;
    }
    SchemaFormComponent.prototype.ngOnChanges = function () {
        this.handleChanges();
    };
    SchemaFormComponent.prototype.ngOnDestroy = function () {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
        }
    };
    /**
     * @description initializes a schema payload imperatively
     */
    SchemaFormComponent.prototype.initSchemaForm = function (schemaForm) {
        var _this = this;
        // Bind them to the component
        this.schemaPayload = schemaForm;
        if (this.schemaPayload) {
            this.groupSubscription = this.schemaPayload.formGroup.valueChanges.subscribe({
                next: function (value) {
                    _this.valueChange.emit(value);
                }
            });
        }
    };
    SchemaFormComponent.prototype.initGroup = function () {
        this.initSchemaForm(new SchemaForm(this.schema, this.data));
    };
    __decorate([
        Input(),
        __metadata("design:type", SchemaForm)
    ], SchemaFormComponent.prototype, "schemaPayload", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], SchemaFormComponent.prototype, "valueChange", void 0);
    __decorate([
        ContentChild(FieldTemplateDirective),
        __metadata("design:type", FieldTemplateDirective)
    ], SchemaFormComponent.prototype, "fieldTemplate", void 0);
    SchemaFormComponent = __decorate([
        Component({
            selector: 'anj-schema-form',
            template: "<ng-container *ngIf=\"schemaPayload && fieldTemplate\">\n    <ng-template [ngTemplateOutlet]=\"fieldTemplate?.template\" [ngTemplateOutletContext]=\"{$implicit: schemaPayload.fields, form: schemaPayload.formGroup, fieldNames: schemaPayload.fieldNames}\"></ng-template>\n</ng-container>\n<ng-content select=\"anj-button-group\"></ng-content>\n",
            changeDetection: ChangeDetectionStrategy.OnPush
        }),
        __metadata("design:paramtypes", [])
    ], SchemaFormComponent);
    return SchemaFormComponent;
}(SchemaControl));

/* @internal */
var SchemaGroup = /** @class */ (function (_super) {
    __extends(SchemaGroup, _super);
    function SchemaGroup() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SchemaGroup.prototype, "groupClass", {
        get: function () { return this._groupClass || ''; },
        set: function (value) {
            this._groupClass = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SchemaGroup.prototype, "fieldClass", {
        get: function () { return this._fieldClass || ''; },
        set: function (value) {
            this._fieldClass = value;
        },
        enumerable: true,
        configurable: true
    });
    SchemaGroup.prototype.handleChanges = function () {
        if (this.group) {
            this.initGroup();
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaGroup.prototype, "groupClass", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaGroup.prototype, "fieldClass", null);
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], SchemaGroup.prototype, "group", void 0);
    __decorate([
        HostBinding('attr.fieldClass'),
        __metadata("design:type", String)
    ], SchemaGroup.prototype, "_fieldClass", void 0);
    __decorate([
        HostBinding('attr.groupClass'),
        __metadata("design:type", String)
    ], SchemaGroup.prototype, "_groupClass", void 0);
    SchemaGroup = __decorate([
        Directive()
    ], SchemaGroup);
    return SchemaGroup;
}(BaseSchema));

var SchemaGroupComponent = /** @class */ (function (_super) {
    __extends(SchemaGroupComponent, _super);
    function SchemaGroupComponent(changeRef, parentGroup) {
        var _this = _super.call(this) || this;
        _this.changeRef = changeRef;
        _this.parentGroup = parentGroup;
        // Define the form group when initialized so that it is defined when needed as a view child property
        _this.formGroup = new FormGroup({});
        _this.valueChange = new EventEmitter();
        return _this;
    }
    Object.defineProperty(SchemaGroupComponent.prototype, "classList", {
        get: function () {
            return "anj-schema-group " + this.groupClass;
        },
        enumerable: true,
        configurable: true
    });
    SchemaGroupComponent.prototype.ngOnChanges = function () {
        this.handleChanges();
    };
    SchemaGroupComponent.prototype.ngOnDestroy = function () {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
        }
    };
    /**
     * @description initializes a schema group imperatively
     */
    SchemaGroupComponent.prototype.initFormGroup = function (_a) {
        var _this = this;
        var _b = __read(_a, 2), fields = _b[0], group = _b[1];
        // Bind them to the component
        this.parsedFields = fields;
        this.formGroup = group;
        // Integrate to existing parent formGroup, if available
        if (this.parentGroup && this.parentGroup.control) {
            Object.keys(group.controls).forEach(function (key) {
                _this.parentGroup.control.addControl(key, group.controls[key]);
            });
            this.formGroup = this.parentGroup.control;
        }
        this.groupSubscription = this.formGroup.valueChanges.subscribe({
            next: function (value) {
                _this.valueChange.emit(value);
            }
        });
    };
    SchemaGroupComponent.prototype.getInputChangeCallback = function (fieldName) {
        return this.onInputChangeCallbacks ? this.onInputChangeCallbacks[fieldName] : undefined;
    };
    SchemaGroupComponent.prototype.initGroup = function () {
        this.initFormGroup(SchemaFormService.parseSchemaPayload(this.group));
    };
    SchemaGroupComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: FormGroupDirective, decorators: [{ type: Optional }] }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SchemaGroupComponent.prototype, "onInputChangeCallbacks", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], SchemaGroupComponent.prototype, "valueChange", void 0);
    __decorate([
        HostBinding('class'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [])
    ], SchemaGroupComponent.prototype, "classList", null);
    SchemaGroupComponent = __decorate([
        Component({
            selector: 'anj-schema-group',
            template: "<ng-container *ngIf=\"parsedFields\">\n    <anj-schema-field *ngFor=\"let field of parsedFields; trackBy: trackByFn;\" [field]=\"field\"\n        [control]=\"formGroup.controls[field.name]\" [inputChangeCallback]=\"getInputChangeCallback(field.name)\"\n        class=\"{{fieldClass}}\" [hideLabels]=\"hideLabels\"></anj-schema-field>\n</ng-container>",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None
        }),
        __param(1, Optional()),
        __metadata("design:paramtypes", [ChangeDetectorRef, FormGroupDirective])
    ], SchemaGroupComponent);
    return SchemaGroupComponent;
}(SchemaGroup));

var SchemaNestedFieldComponent = /** @class */ (function (_super) {
    __extends(SchemaNestedFieldComponent, _super);
    function SchemaNestedFieldComponent() {
        return _super.call(this) || this;
    }
    Object.defineProperty(SchemaNestedFieldComponent.prototype, "label", {
        get: function () {
            return this._label || '';
        },
        set: function (value) {
            this._label = value;
        },
        enumerable: true,
        configurable: true
    });
    SchemaNestedFieldComponent.prototype.updateNestedField = function (newValue) {
        this.control.patchValue(newValue);
    };
    __decorate([
        Input(),
        __metadata("design:type", AbstractControl)
    ], SchemaNestedFieldComponent.prototype, "control", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaNestedFieldComponent.prototype, "label", null);
    __decorate([
        HostBinding('class.anj-schema-nested-field'),
        __metadata("design:type", Boolean)
    ], SchemaNestedFieldComponent.prototype, "isField", void 0);
    SchemaNestedFieldComponent = __decorate([
        Component({
            selector: 'anj-schema-nested-field',
            template: "<anj-form-group>\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<div class=\"anj-nested\">\n    <ng-container [formGroup]=\"control\">\n        <anj-schema-field *ngFor=\"let nestedField of field.parsedFields; trackBy: trackByFn;\"\n                          [field]=\"nestedField\"\n                          [control]=\"control['controls'][nestedField.name]\"\n        ></anj-schema-field>\n    </ng-container>\n</div>\n\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            styles: ["anj-schema-nested-field .anj-nested{margin:0 0 1rem 1rem}anj-schema-nested-field .anj-form-group.anj-has-value{margin:0}"]
        }),
        __metadata("design:paramtypes", [])
    ], SchemaNestedFieldComponent);
    return SchemaNestedFieldComponent;
}(SchemaField));

var ReadFieldComponent = /** @class */ (function (_super) {
    __extends(ReadFieldComponent, _super);
    function ReadFieldComponent(element, renderer) {
        var _this = _super.call(this) || this;
        _this.element = element;
        _this.renderer = renderer;
        return _this;
    }
    Object.defineProperty(ReadFieldComponent.prototype, "fieldItemPropertiesKeys", {
        get: function () {
            return SchemaFormService.parseFieldItemKeys(this.field);
        },
        enumerable: true,
        configurable: true
    });
    ReadFieldComponent.prototype.ngAfterViewInit = function () {
        this.renderer.addClass(this.element.nativeElement, this.field.name.split(' ').join('_'));
    };
    ReadFieldComponent.prototype.toDisplayValue = function () {
        var displayValue = this.value ? this.value : '';
        if (Array.isArray(this.value)) {
            displayValue = this.value.join(', ');
        }
        return displayValue;
    };
    ReadFieldComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ReadFieldComponent.prototype, "value", void 0);
    __decorate([
        HostBinding('class.anj-read-field'),
        __metadata("design:type", Boolean)
    ], ReadFieldComponent.prototype, "isField", void 0);
    ReadFieldComponent = __decorate([
        Component({
            selector: 'anj-read-field',
            template: "<ng-container [ngSwitch]=\"field.toInputType()\">\n    <span *ngIf=\"!hideLabels\" class=\"anj-schema-label\">\n        {{field.label}}\n    </span>\n    <span class=\"anj-schema-display-value\" [ngClass]=\"{'nested': field.toInputType() === 'nested'}\">\n        <ng-template ngSwitchCase=\"date\">\n            {{value | date: 'shortDate'}}\n        </ng-template>\n        <ng-template ngSwitchCase=\"checkbox\">\n            <svg *ngIf=\"value\" xmlns=\"http://www.w3.org/2000/svg\" width=\"18\" height=\"18\" viewBox=\"0 0 18 18\">\n                <path d=\"M6.61 11.89L3.5 8.78 2.44 9.84 6.61 14l8.95-8.95L14.5 4z\"/>\n            </svg>\n        </ng-template>\n        <ng-template ngSwitchCase=\"table\">\n            <anj-table [dataSource]=\"value\">\n                <anj-column *ngFor=\"let key of fieldItemPropertiesKeys;\" [field]=\"key\"></anj-column>\n            </anj-table>\n        </ng-template>\n        <ng-template ngSwitchCase=\"nested\">\n            <anj-schema-read\n                [group]=\"[field.nestedDef, value]\"\n            ></anj-schema-read>\n        </ng-template>\n        <ng-template ngSwitchDefault>\n            {{toDisplayValue()}}\n        </ng-template>\n    </span>\n</ng-container>\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            styles: [".anj-read-field{position:relative}.anj-read-field .nested{margin-left:1rem}.anj-read-field .anj-schema-label{color:var(--anj-neutral);font-size:.75rem;line-height:1;margin-bottom:0;position:relative;top:0}.anj-read-field .anj-schema-display-value{display:block;min-height:1.75rem}"]
        }),
        __metadata("design:paramtypes", [ElementRef, Renderer2])
    ], ReadFieldComponent);
    return ReadFieldComponent;
}(SchemaField));

var SchemaReadComponent = /** @class */ (function (_super) {
    __extends(SchemaReadComponent, _super);
    function SchemaReadComponent() {
        return _super.call(this) || this;
    }
    Object.defineProperty(SchemaReadComponent.prototype, "classList", {
        get: function () {
            return "anj-schema-read " + this.groupClass;
        },
        enumerable: true,
        configurable: true
    });
    SchemaReadComponent.prototype.ngOnChanges = function () {
        this.handleChanges();
    };
    SchemaReadComponent.prototype.initGroup = function () {
        this.parsedFields = SchemaFormService.parseSchema(this.group[0]);
        this.values = this.group[1] || {};
    };
    __decorate([
        HostBinding('class'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [])
    ], SchemaReadComponent.prototype, "classList", null);
    SchemaReadComponent = __decorate([
        Component({
            selector: 'anj-schema-read',
            template: "<ng-container *ngIf=\"parsedFields\">\n    <anj-read-field *ngFor=\"let field of parsedFields\"\n        [field]=\"field\"\n        [value]=\"values[field.name]\"\n        class=\"{{fieldClass}}\"\n        [hideLabels]=\"hideLabels\"\n    ></anj-read-field>\n</ng-container>\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None
        }),
        __metadata("design:paramtypes", [])
    ], SchemaReadComponent);
    return SchemaReadComponent;
}(SchemaGroup));

/* @internal */
var SchemaArray = /** @class */ (function (_super) {
    __extends(SchemaArray, _super);
    function SchemaArray() {
        return _super.call(this) || this;
    }
    Object.defineProperty(SchemaArray.prototype, "controlArrayValue", {
        get: function () {
            return this.control.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SchemaArray.prototype, "formArray", {
        get: function () {
            return this.control;
        },
        enumerable: true,
        configurable: true
    });
    SchemaArray.prototype.addRow = function (additionalRows, insertPosition) {
        if (additionalRows === void 0) { additionalRows = 1; }
        for (var i = 0; i < additionalRows; i++) {
            var insertIndex = insertPosition !== undefined ? insertPosition : this.controlArrayValue.length;
            this.formArray.insert(insertIndex, this.newItem());
        }
        this.acknowledgeChange();
    };
    SchemaArray.prototype.removeRow = function (index) {
        this.formArray.removeAt(index);
        this.acknowledgeChange();
    };
    SchemaArray.prototype.atMaxItems = function () {
        return this.field.maxItems && (this.control.length === this.field.maxItems);
    };
    SchemaArray.prototype.atMinItems = function () {
        return this.field.minItems && (this.control.length === this.field.minItems);
    };
    SchemaArray.prototype.showToolTip = function (event, popper, content) {
        clearTimeout(this.timer);
        this.content = content;
        popper.display(event.target);
    };
    SchemaArray.prototype.hideToolTip = function (popper) {
        this.timer = window.setTimeout(function () {
            popper.hide();
        }, 750);
    };
    SchemaArray.prototype.setArray = function () {
        // If minItems and data is array shorter than minItems, add more empty rows
        if (this.field.minItems &&
            (Array.isArray(this.controlArrayValue) && this.controlArrayValue.length < this.field.minItems)) {
            var additionalRows = this.field.minItems - this.controlArrayValue.length;
            this.addRow(additionalRows);
            // If Data is not an array, add minimum rows (minItems or 1)
        }
        else if (!Array.isArray(this.controlArrayValue)) {
            var additionalRows = (this.field.minItems > 0) ? this.field.minItems : 1;
            this.addRow(additionalRows);
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", FormArray)
    ], SchemaArray.prototype, "control", void 0);
    SchemaArray = __decorate([
        Directive(),
        __metadata("design:paramtypes", [])
    ], SchemaArray);
    return SchemaArray;
}(SchemaField));

var SchemaTableComponent = /** @class */ (function (_super) {
    __extends(SchemaTableComponent, _super);
    function SchemaTableComponent(changeRef) {
        var _this = _super.call(this) || this;
        _this.changeRef = changeRef;
        return _this;
    }
    Object.defineProperty(SchemaTableComponent.prototype, "label", {
        get: function () {
            return this._label || '';
        },
        set: function (value) {
            this._label = value;
        },
        enumerable: true,
        configurable: true
    });
    // Listen to rows added on anj-table, modify the NG Control respectively
    SchemaTableComponent.prototype._onRowAdd = function (event) {
        this.addRow(1, event.detail.index);
    };
    // Listen to rows removed on anj-table, modify the NG Control respectively
    SchemaTableComponent.prototype._onRowRemove = function (event) {
        this.removeRow(event.detail.index);
    };
    Object.defineProperty(SchemaTableComponent.prototype, "fieldItemPropertiesKeys", {
        get: function () {
            return SchemaFormService.parseFieldItemKeys(this.field);
        },
        enumerable: true,
        configurable: true
    });
    SchemaTableComponent.prototype.ngOnInit = function () {
        this.localControlValue = this.controlArrayValue;
        this.setArray();
    };
    SchemaTableComponent.prototype.ngOnChanges = function (changes) {
        this.acknowledgeChange();
    };
    SchemaTableComponent.prototype.getDataType = function (def) {
        // TODO: we probably need to handle at least numbers too
        if (def.type === 'boolean') {
            return 'boolean';
        }
    };
    SchemaTableComponent.prototype.findIndex = function (fields, c) {
        return fields.findIndex(function (t) { return t.name === c.field; });
    };
    SchemaTableComponent.prototype.newItem = function () {
        return SchemaFormService.generateFormGroup(this.field.parsedArray, {});
    };
    SchemaTableComponent.prototype.acknowledgeChange = function () {
        this.localControlValue = this.controlArrayValue.slice(0);
        this.changeRef.markForCheck();
    };
    SchemaTableComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaTableComponent.prototype, "label", null);
    __decorate([
        HostBinding('class.anj-schema-table'),
        __metadata("design:type", Boolean)
    ], SchemaTableComponent.prototype, "isField", void 0);
    __decorate([
        HostListener('anjCreateStart', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [CustomEvent]),
        __metadata("design:returntype", void 0)
    ], SchemaTableComponent.prototype, "_onRowAdd", null);
    __decorate([
        HostListener('anjDelete', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [CustomEvent]),
        __metadata("design:returntype", void 0)
    ], SchemaTableComponent.prototype, "_onRowRemove", null);
    SchemaTableComponent = __decorate([
        Component({
            selector: 'anj-schema-table',
            template: "<anj-form-group class=\"anj-schema-table-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<anj-table [dataSource]=\"localControlValue\" column-width=\"auto-fix\" height=\"400px\">\n    <anj-editing\n        mode=\"batch\"\n        creating=\"true\"\n        deleting=\"true\"\n        saving=\"false\"\n        [deleteDisabled]=\"atMinItems()\"\n        [createDisabled]=\"atMaxItems()\"\n    ></anj-editing>\n    <anj-column *ngFor=\"let key of fieldItemPropertiesKeys; let index = index;\"\n                [dataType]=\"getDataType(field.parsedArray[index])\"\n                [field]=\"key\"\n                sortable=\"false\"\n                edit-template=\"controlTemplate\"\n    ></anj-column>\n    <ng-container *anjTemplate=\"let row of 'controlTemplate'; let i = index; let c = column;\">\n        <anj-schema-field\n            *ngIf=\"control.controls[i]\"\n            [field]=\"field.parsedArray[findIndex(field.parsedArray, c)]\"\n            [control]=\"control.controls[i]['controls'][c.field]\"\n            [hideLabels]=\"true\">\n        </anj-schema-field>\n    </ng-container>\n</anj-table>\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            styles: ["anj-schema-table .anj-schema-table-label{margin-top:1rem}anj-schema-table anj-schema-field{display:block}"]
        }),
        __metadata("design:paramtypes", [ChangeDetectorRef])
    ], SchemaTableComponent);
    return SchemaTableComponent;
}(SchemaArray));

var SchemaContextComponent = /** @class */ (function (_super) {
    __extends(SchemaContextComponent, _super);
    function SchemaContextComponent() {
        var _this = _super.call(this) || this;
        _this.valueChange = new EventEmitter();
        return _this;
    }
    Object.defineProperty(SchemaContextComponent.prototype, "groupClass", {
        get: function () { return this._groupClass || ''; },
        set: function (value) {
            this._groupClass = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SchemaContextComponent.prototype, "fieldClass", {
        get: function () { return this._fieldClass || ''; },
        set: function (value) {
            this._fieldClass = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SchemaContextComponent.prototype, "mode", {
        get: function () { return this._mode || 'view'; },
        set: function (value) {
            this._mode = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SchemaContextComponent.prototype, "classList", {
        get: function () {
            return "anj-schema-context anj-schema-context-" + this.context;
        },
        enumerable: true,
        configurable: true
    });
    SchemaContextComponent.prototype.updateApp = function (event) {
        this.valueChange.emit(event);
    };
    SchemaContextComponent.prototype.ngOnChanges = function () {
        this.schemaPayload = this.combinePayload();
    };
    SchemaContextComponent.prototype.combinePayload = function () {
        // Create Mode
        if (this.mode === 'create' && this.schema) {
            return [this.schema, null];
            // View Mode
        }
        else if (this.mode === 'view' && this.schema && !this.data) {
            return [this.schema, {}];
            // Update Mode
        }
        else if (this.schema && this.data) {
            return [this.schema, this.data];
        }
        return null;
    };
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaContextComponent.prototype, "groupClass", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaContextComponent.prototype, "fieldClass", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], SchemaContextComponent.prototype, "mode", null);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SchemaContextComponent.prototype, "schema", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SchemaContextComponent.prototype, "data", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SchemaContextComponent.prototype, "context", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], SchemaContextComponent.prototype, "valueChange", void 0);
    __decorate([
        ViewChild(SchemaGroupComponent),
        __metadata("design:type", SchemaGroupComponent)
    ], SchemaContextComponent.prototype, "schemaGroup", void 0);
    __decorate([
        HostBinding('class'),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [])
    ], SchemaContextComponent.prototype, "classList", null);
    __decorate([
        HostBinding('attr.fieldClass'),
        __metadata("design:type", String)
    ], SchemaContextComponent.prototype, "_fieldClass", void 0);
    __decorate([
        HostBinding('attr.groupClass'),
        __metadata("design:type", String)
    ], SchemaContextComponent.prototype, "_groupClass", void 0);
    __decorate([
        HostBinding('attr.mode'),
        __metadata("design:type", String)
    ], SchemaContextComponent.prototype, "_mode", void 0);
    SchemaContextComponent = __decorate([
        Component({
            selector: 'anj-schema-context, anj-custom-fields',
            template: "<ng-container *ngIf=\"mode === 'view'; else formFields\">\n    <anj-schema-read\n        [group]=\"schemaPayload\"\n        [groupClass]=\"groupClass\"\n        [fieldClass]=\"fieldClass\"\n        [hideLabels]=\"hideLabels\"\n    ></anj-schema-read>\n</ng-container>\n<ng-template #formFields>\n    <anj-schema-group\n        [group]=\"schemaPayload\"\n        [groupClass]=\"groupClass\"\n        [fieldClass]=\"fieldClass\"\n        [hideLabels]=\"hideLabels\"\n        (valueChange)=\"updateApp($event)\"\n    ></anj-schema-group>\n</ng-template>\n",
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None
        }),
        __metadata("design:paramtypes", [])
    ], SchemaContextComponent);
    return SchemaContextComponent;
}(BaseSchema));

var translations = {
    en: {
        'is required.': 'is required.',
        'is not a valid date.': 'is not a valid date.',
        'Please select an option.': 'Please select an option.',
        'must be at least': 'must be at least',
        'must not be greater than': 'must not be greater than',
        'must not have duplicate values.': 'must not have duplicate values.',
        'must have at least': 'must have at least',
        'entries.': 'entries.',
        'characters.': 'characters.',
        'must adhere to a specific pattern': 'must adhere to a specific pattern',
        'must adhere to an email pattern (ex. name@domain.com)': 'must adhere to an email pattern (ex. name@domain.com)',
        'Add an item below': 'Add an item below',
        'Remove this item': 'Remove this item',
        'Add a row below': 'Add a row below',
        'Remove this row': 'Remove this row',
    },
    fr: {
        'is required.': 'est obligatoire.',
        'is not a valid date.': 'n’est pas une date valide.',
        'Please select an option.': 'Sélectionnez une option.',
        'must be at least': 'doit être au moins',
        'must not be greater than': 'ne doit pas dépasser',
        'must not have duplicate values.': 'ne doit pas inclure de valeurs dupliquées.',
        'must have at least': 'doit inclure au moins',
        'entries.': 'entrées.',
        'characters.': 'caractères.',
        'must adhere to a specific pattern': 'doit correspondre à un format spécifique',
        'must adhere to an email pattern (ex. name@domain.com)': 'doit correspondre à un format d’e-mail (par ex., nom@domaine.com)',
        'Add an item below': 'Ajouter un élément ci-dessous',
        'Remove this item': 'Supprimer cet élément',
        'Add a row below': 'Ajouter une ligne ci-dessous',
        'Remove this row': 'Supprimer cette ligne',
    },
    de: {
        'is required.': 'ist erforderlich.',
        'is not a valid date.': 'ist kein gültiges Datum.',
        'Please select an option.': 'Wählen Sie eine Option aus.',
        'must be at least': 'muss mindestens folgenden Wert haben:',
        'must not be greater than': 'darf nicht größer als folgender Wert sein:',
        'must not have duplicate values.': 'darf keine duplizierten Werte aufweisen.',
        'must have at least': 'muss mindestens',
        'entries.': 'Einträge aufweisen.',
        'characters.': 'Zeichen lang sein.',
        'must adhere to a specific pattern': 'muss ein bestimmtes Muster befolgen',
        'must adhere to an email pattern (ex. name@domain.com)': 'muss ein bestimmtes E-Mail-Muster befolgen (Bsp.: name@domain.com)',
        'Add an item below': 'Element unterhalb hinzufügen',
        'Remove this item': 'Dieses Element entfernen',
        'Add a row below': 'Zeile unterhalb hinzufügen',
        'Remove this row': 'Diese Zeile entfernen',
    },
    ja: {
        'is required.': 'は必須です。',
        'is not a valid date.': 'は有効な日付ではありません。',
        'Please select an option.': 'オプションを選択してください。',
        'must be at least': 'の最小長さは',
        'must not be greater than': 'の最大長さは',
        'must not have duplicate values.': 'の値は重複させることはできません。',
        'must have at least': 'には少なくとも',
        'entries.': '個のエントリが必要です。',
        'characters.': '文字にする必要があります。',
        'must adhere to a specific pattern': 'は指定のパターン',
        'must adhere to an email pattern (ex. name@domain.com)': 'は電子メールのパターン (name@domain.com など) に従う必要があります',
        'Add an item below': '以下のアイテムを追加',
        'Remove this item': 'このアイテムを削除',
        'Add a row below': '以下の行を追加',
        'Remove this row': 'この行を削除',
    },
    es: {
        'is required.': 'es obligatoria.',
        'is not a valid date.': 'no es una fecha válida',
        'Please select an option': 'Por favor, seleccione una opción',
        'must be at least': 'debe ser al menos',
        'must not be greater than': 'no debe ser mayor que',
        'must not have duplicate values': 'no debe contener valores duplicados',
        'must have at least': 'debe tener al menos',
        'entries.': 'entradas.',
        'characters.': 'caracteres.',
        'must adhere to a specific pattern': 'debe ajustarse a un patrón específico',
        'must adhere to an email pattern (ex. name@domain.com)': 'debe adherirse a un patrón de correo electrónico (por ejemplo, nombre@dominio.com)',
        'Add an item below': 'Agregar un elemento abajo',
        'Remove this item': 'Eliminar este elemento',
        'Add a row below': 'Agregar una fila abajo',
        'Remove this row': 'Eliminar esta fila'
    }
};

/** @internal */
var SchemaTranslationPipe = /** @class */ (function (_super) {
    __extends(SchemaTranslationPipe, _super);
    function SchemaTranslationPipe() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.translations = translations;
        return _this;
    }
    SchemaTranslationPipe = __decorate([
        Pipe({
            name: 'anjSchemaTranslate'
        })
    ], SchemaTranslationPipe);
    return SchemaTranslationPipe;
}(BaseTranslationPipe));

var SchemaTierComponent = /** @class */ (function (_super) {
    __extends(SchemaTierComponent, _super);
    function SchemaTierComponent(changeRef, el) {
        var _this = _super.call(this, changeRef) || this;
        _this.el = el;
        _this.uom = { value: 1 };
        return _this;
    }
    SchemaTierComponent.prototype.ngOnInit = function () {
        var _this = this;
        _super.prototype.ngOnInit.call(this);
        console.log(this.field, '<<');
        this.uoms = (this.field.uoms)
            ? this.field.uoms.map(function (f) { return ({ name: f.name, value: Number.parseFloat(f.value) }); })
            : undefined;
        var initialSchemaUOM = this.uoms.find(function (u) { return u.name === _this.field.uom; });
        this.uom = initialSchemaUOM || { value: 1 };
        this.refreshTiers();
    };
    SchemaTierComponent.prototype._onRowAdd = function (event) {
        var length = this.control.length;
        this.addRow(1, length);
    };
    // Passthrough to manage the readonly features of the first column (sparing the first row)
    SchemaTierComponent.prototype.getField = function (column, rowIndex) {
        var columnIndex = this.findIndex(this.field.parsedArray, column);
        var field = this.field.parsedArray[columnIndex];
        field['readonly'] = !!(columnIndex === 0 && rowIndex > 0);
        return field;
    };
    SchemaTierComponent.prototype.tierToChange = function (index) {
        var nextRow = this.control.at(index + 1);
        var currentRowTierTo = this.control.at(index).get('tierTo').value;
        if (nextRow) {
            nextRow.patchValue({ tierFrom: currentRowTierTo + this.uom.value });
        }
    };
    SchemaTierComponent.prototype.setUom = function (_a) {
        var detail = _a.detail;
        this.uom = detail.value;
        var domEvent = new CustomEvent('uomChange', {
            detail: this.uom,
            bubbles: true
        });
        this.el.nativeElement.dispatchEvent(domEvent);
        this.refreshTiers();
    };
    SchemaTierComponent.prototype.newItem = function () {
        var group = SchemaFormService.generateFormGroup(this.field.parsedArray, {});
        var previousTier = this.control.at(this.control.length - 1);
        var previousTierTo = previousTier ? Number.parseFloat(previousTier.get('tierTo').value) : 0;
        group.patchValue({ tierFrom: previousTierTo + this.uom.value });
        return group;
    };
    SchemaTierComponent.prototype.refreshTiers = function () {
        var controls = this.control.controls;
        for (var i = 0; i < controls.length; i++) {
            if (i > 0) {
                var previousTier = controls[i - 1].value;
                var endingUnit = Number.parseFloat(previousTier.tierTo);
                controls[i].patchValue({ tierFrom: endingUnit + this.uom.value });
            }
        }
        this.acknowledgeChange();
    };
    SchemaTierComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: ElementRef }
    ]; };
    SchemaTierComponent = __decorate([
        Component({
            selector: 'anj-schema-tier',
            template: "<anj-form-group class=\"anj-schema-table-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<anj-table [dataSource]=\"localControlValue\" column-width=\"auto-fix\" height=\"400px\">\n    <anj-editing\n        mode=\"batch\"\n        creating=\"true\"\n        deleting=\"true\"\n        saving=\"false\"\n        [deleteDisabled]=\"atMinItems()\"\n        [createDisabled]=\"atMaxItems()\"\n    ></anj-editing>\n    <anj-column *ngFor=\"let key of fieldItemPropertiesKeys; let index = index;\"\n                [dataType]=\"getDataType(field.parsedArray[index])\"\n                [field]=\"key\"\n                sortable=\"false\"\n                edit-template=\"controlTemplate\"\n    ></anj-column>\n    <ng-container *anjTemplate=\"let row of 'controlTemplate'; let i = index; let c = column;\">\n        <anj-schema-field\n            *ngIf=\"control.controls[i]\"\n            [field]=\"getField(c, i)\"\n            [control]=\"control.controls[i]['controls'][c.field]\"\n            [index]=\"i\"\n            [hideLabels]=\"true\"\n            (tierToChange)=\"tierToChange($event)\">\n        </anj-schema-field>\n    </ng-container>\n</anj-table>\n<anj-form-group *ngIf=\"uoms\">\n    <anj-label>Unit of Measurement</anj-label>\n    <anj-select (anjChange)=\"setUom($event)\" [value]=\"uom\">\n        <anj-option *ngFor=\"let u of uoms\" [value]=\"u\">{{u.name}}</anj-option>\n    </anj-select>\n</anj-form-group>\n",
            styles: ["anj-schema-table .anj-schema-table-label{margin-top:1rem}anj-schema-table anj-schema-field{display:block}"]
        }),
        __metadata("design:paramtypes", [ChangeDetectorRef, ElementRef])
    ], SchemaTierComponent);
    return SchemaTierComponent;
}(SchemaTableComponent));

var AnjunaMetaUIModule = /** @class */ (function () {
    function AnjunaMetaUIModule() {
    }
    AnjunaMetaUIModule = __decorate([
        NgModule({
            declarations: [
                FieldTemplateDirective,
                SchemaFieldComponent,
                SchemaFormComponent,
                SchemaGroupComponent,
                SchemaNestedFieldComponent,
                SchemaTranslationPipe,
                SchemaReadComponent,
                SchemaTableComponent,
                ReadFieldComponent,
                SchemaContextComponent,
                SchemaTierComponent
            ],
            imports: [
                CommonModule,
                ReactiveFormsModule,
                FormsModule,
                AnjunaCoreModule
            ],
            exports: [
                FieldTemplateDirective,
                SchemaFieldComponent,
                SchemaFormComponent,
                SchemaGroupComponent,
                SchemaNestedFieldComponent,
                SchemaReadComponent,
                SchemaTableComponent,
                ReadFieldComponent,
                SchemaContextComponent
            ],
            schemas: [CUSTOM_ELEMENTS_SCHEMA]
        })
    ], AnjunaMetaUIModule);
    return AnjunaMetaUIModule;
}());

/* Components */

/**
 * Generated bundle index. Do not edit.
 */

export { AnjunaMetaUIModule, FieldTemplateDirective, ParsedField, ReadFieldComponent, SchemaContextComponent, SchemaFieldComponent, SchemaForm, SchemaFormComponent, SchemaFormService, SchemaGroupComponent, SchemaNestedFieldComponent, SchemaReadComponent, SchemaTableComponent, SchemaField as ɵa, BaseSchema as ɵb, SchemaControl as ɵc, SchemaGroup as ɵd, SchemaArray as ɵe, SchemaTranslationPipe as ɵf, SchemaTierComponent as ɵg };
//# sourceMappingURL=anjuna-angular-meta-ui.js.map
